#include "bsp.h"
#include "mrfi.h"
#include "nwk_types.h"
#include "nwk_api.h"
#include "bsp_leds.h"
#include "bsp_buttons.h"
#include "vlo_rand.h"

void MCU_Init(void);
void read_flash(int num_mes);
void write_flash(uint8_t * value, int num_mes);
uint8_t val[3];
void erase_flash(void);
linkID_t linkID1; 
int nbr_mes=0;
__no_init volatile int tempOffset @ 0x10F4; // Temperature offset set at production
__no_init volatile char Flash_Addr[4] @ 0x10F0; // Flash address set randomly

void createRandomAddress(); 

void main (void)
{
  addr_t lAddr;
  WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT
  { 
    volatile int i;
    for(i = 0; i < 0xFFFF; i++){}
  }
  if( CALBC1_8MHZ == 0xFF )                 // Do not run if cal values are erased
  {
    volatile int i;
    P1DIR |= 0x03;
    BSP_TURN_ON_LED1();
    BSP_TURN_OFF_LED2();
    while(1)
    {
      for(i = 0; i < 0x5FFF; i++){}
      BSP_TOGGLE_LED2();
      BSP_TOGGLE_LED1();
    }
  }
  
 
  
  P1DIR |= 0x03 ;        // config BP 
  P1DIR &= ~0x04 ;
  P1REN |= 0x04 ;
  P1IE |= 0x04 ; 
  
  P2DIR = 0x27;
  P2OUT = 0x00;
  P3DIR = 0xC0;
  P3OUT = 0x00;
  P4DIR = 0xFF;
  P4OUT = 0x00;
  
  BSP_Init();
  
  if( Flash_Addr[0] == 0xFF && 
      Flash_Addr[1] == 0xFF && 
      Flash_Addr[2] == 0xFF && 
      Flash_Addr[3] == 0xFF )
  {
    createRandomAddress();                  // set Random device address at initial startup
  }
  lAddr.addr[0]=Flash_Addr[0];
  lAddr.addr[1]=Flash_Addr[1];
  lAddr.addr[2]=Flash_Addr[2];
  lAddr.addr[3]=Flash_Addr[3];
  SMPL_Ioctl(IOCTL_OBJ_ADDR, IOCTL_ACT_SET, &lAddr);
  

  BCSCTL1 = CALBC1_8MHZ;                    // Set DCO after random function
  DCOCTL = CALDCO_8MHZ;  
   
  
  TACCR0 = 30;                              // Delay to allow Ref to settle
  TACCTL0 &= ~CCIE;                         
  TACTL = TASSEL_2 + MC_1;                  // TACLK = SMCLK, Up mode
                      
  
  
  while (SMPL_NO_JOIN == SMPL_Init((uint8_t (*)(linkID_t))0))
  {
    BSP_TOGGLE_LED1();
    BSP_TOGGLE_LED2();
    __bis_SR_register(LPM0_bits + GIE);     // LPM3 with interrupts enabled
  }
  

   while (SMPL_SUCCESS != SMPL_Link(&linkID1))
  {
    __bis_SR_register(LPM0_bits + GIE);     
    BSP_TOGGLE_LED1();
    BSP_TOGGLE_LED2();
  }
   
  // Turn off all LEDs
  if (BSP_LED1_IS_ON()) BSP_TOGGLE_LED1();
  if (BSP_LED2_IS_ON()) BSP_TOGGLE_LED2();
  

  __bis_SR_register(LPM0_bits + GIE);
  while (1);
  
}

void createRandomAddress()
{
  unsigned int rand, rand2;
  do
  {
    rand = TI_getRandomIntegerFromVLO();    // first byte can not be 0x00 of 0xFF
  }
  while( (rand & 0xFF00)==0xFF00 || (rand & 0xFF00)==0x0000 );
  rand2 = TI_getRandomIntegerFromVLO();
  
  BCSCTL1 = CALBC1_8MHZ;                    // Set DCO to 1MHz
  DCOCTL = CALDCO_8MHZ;
  FCTL2 = FWKEY + FSSEL0 + FN1;             // MCLK/3 for Flash Timing Generator
  FCTL3 = FWKEY + LOCKA;                    // Clear LOCK & LOCKA bits
  FCTL1 = FWKEY + WRT;                      // Set WRT bit for write operation

  Flash_Addr[0]=(rand>>8) & 0xFF;
  Flash_Addr[1]=rand & 0xFF;
  Flash_Addr[2]=(rand2>>8) & 0xFF; 
  Flash_Addr[3]=rand2 & 0xFF; 
  
  FCTL1 = FWKEY;                            // Clear WRT bit
  FCTL3 = FWKEY + LOCKA + LOCK;             // Set LOCK & LOCKA bit
}


void erase_flash () // erase segments B, C & D
{  
       char *Flash_ptrB,*Flash_ptrC,*Flash_ptrD;
       
       Flash_ptrB = (char *)0x1080; 
       Flash_ptrC = (char *)0x1040;              // Initialize Flash segment C ptr
       Flash_ptrD = (char *)0x1000; 
       
       FCTL3 = FWKEY;  
       FCTL1 = FWKEY + ERASE ;              // Set Erase bit, allow interrupts
     
       *Flash_ptrB = 0;
       *Flash_ptrC = 0;
       *Flash_ptrD = 0;
       
       FCTL1 = FWKEY;
       FCTL3 = FWKEY + LOCK; 
}

void write_flash (uint8_t* value,int num_mes)
{
       int i;
       char *Flash_ptr; 
       Flash_ptr = (char *)0x1000 + num_mes*3;       
      
       FCTL3 = FWKEY;       
       FCTL1 = FWKEY + WRT;       
 
       for(i=0;i<3;i++) *Flash_ptr++=value[i];           

       FCTL1 = FWKEY;
       FCTL3 = FWKEY + LOCK;                 
}  

void read_flash (int num_mes)
{ 
       int i;
       char *Flash_ptr;       
       Flash_ptr = (char *)0x1000 + num_mes*3;
       
       FCTL1 = FWKEY; 
       FCTL3 = FWKEY;               
       
       for(i=0;i<3;i++) val[i]=*Flash_ptr++;           

       FCTL1 = FWKEY;
       FCTL3 = FWKEY + LOCK;       
} 


#pragma vector=PORT1_VECTOR
__interrupt void Port_1(void)
{
   
    volatile long temp;
    int degC, volt,i,j;
    int results[2];
    int nbr_mes_max=3;
    uint8_t  msg[3];

    P1IFG = 0;
   
   
    ADC10CTL1 = INCH_10 + ADC10DIV_4;       // Temp Sensor ADC10CLK/5
    ADC10CTL0 = SREF_1 + ADC10SHT_3 + REFON + ADC10ON + ADC10SR ;
    for( degC = 240; degC > 0; degC-- );    // delay to allow reference to settle
    ADC10CTL0 |= ENC + ADC10SC;             // Sampling and conversion start
    while (ADC10CTL1 & ADC10BUSY);
    results[0] = ADC10MEM;    
    ADC10CTL0 &= ~ENC; 
    
    ADC10CTL1 = INCH_11;                     // AVcc/2
    ADC10CTL0 = SREF_1 + ADC10SHT_2 + REFON + ADC10ON + REF2_5V ;
    for( degC = 240; degC > 0; degC-- );    // delay to allow reference to settle
    ADC10CTL0 |= ENC + ADC10SC;             // Sampling and conversion start
    while (ADC10CTL1 & ADC10BUSY);
    results[1] = ADC10MEM;
    ADC10CTL0 &= ~ENC;
    ADC10CTL0 &= ~(REFON + ADC10ON);        // turn off A/D to save power
    
    temp = results[0];
    degC = ((temp - 673) * 4230) / 1024;
    if( tempOffset != 0xFFFF )
    {
      degC += tempOffset; 
    }
     temp = results[1];
     volt = (temp*25)/512;
     msg[0] = degC&0xFF;
     msg[1] = (degC>>8)&0xFF;
     msg[2] = volt;
  
         
      nbr_mes++;       
      write_flash(msg,nbr_mes-1);
      
      if(nbr_mes == nbr_mes_max)
      {       
          P1IFG = 0;
          P1IE &= ~0x04; 
         
          for(j=1;j<=nbr_mes_max;j++) 
          {                  
            for(i=0;i<3;i++) val[i]=0; 
            read_flash(j-1);              
            for(i=0;i<3;i++) msg[i]=val[i];
            if (SMPL_SUCCESS == SMPL_Send(linkID1, msg, sizeof(msg)))
            BSP_TOGGLE_LED2();
            for(i = 0; i < 0xFFFF; i++){}
          }   
          nbr_mes=0;
          P1IE |= 0x04;
          
          
            __delay_cycles(3000);  
          __bis_SR_register(CPUOFF + GIE);
      }            
}